@@ -119,7 +119,8 @@ struct _GeditPreferencesDialog
 
 	GtkWidget	*display_line_numbers_checkbutton;
 	GtkWidget	*display_statusbar_checkbutton;
-	GtkWidget       *display_grid_checkbutton;
+	GtkWidget	*display_overview_map_checkbutton;
+	GtkWidget	*display_grid_checkbutton;
 
 	/* Right margin */
 	GtkWidget	*right_margin_checkbutton;
@@ -184,6 +185,7 @@ gedit_preferences_dialog_class_init (GeditPreferencesDialogClass *klass)
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, display_line_numbers_checkbutton);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, display_statusbar_checkbutton);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, display_grid_checkbutton);
+	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, display_overview_map_checkbutton);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, right_margin_checkbutton);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, right_margin_position_grid);
 	gtk_widget_class_bind_template_child (widget_class, GeditPreferencesDialog, right_margin_position_spinbutton);
@@ -336,10 +338,10 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	background_pattern = g_settings_get_enum (dlg->editor,
 	                                          GEDIT_SETTINGS_BACKGROUND_PATTERN);
 
-	/* Set initial state */
 	wrap_mode = g_settings_get_enum (dlg->editor,
 					 GEDIT_SETTINGS_WRAP_MODE);
 
+	/* Set initial state */
 	switch (wrap_mode)
 	{
 		case GTK_WRAP_WORD:
@@ -385,7 +387,7 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	/* Set widgets sensitivity */
 	gtk_widget_set_sensitive (dlg->split_checkbutton,
 				  (wrap_mode != GTK_WRAP_NONE));
-	/* Connect signals */
+
 	g_settings_bind (dlg->editor,
 			 GEDIT_SETTINGS_DISPLAY_LINE_NUMBERS,
 			 dlg->display_line_numbers_checkbutton,
@@ -401,6 +403,11 @@ setup_view_page (GeditPreferencesDialog *dlg)
 	                 dlg->display_statusbar_checkbutton,
 	                 "active",
 	                 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
+	g_settings_bind (dlg->editor,
+			 GEDIT_SETTINGS_DISPLAY_OVERVIEW_MAP,
+			 dlg->display_overview_map_checkbutton,
+			 "active",
+			 G_SETTINGS_BIND_GET | G_SETTINGS_BIND_SET);
 	g_settings_bind (dlg->editor,
 	                 GEDIT_SETTINGS_DISPLAY_RIGHT_MARGIN,
 	                 dlg->right_margin_checkbutton,
@@ -591,7 +598,7 @@ file_copy (const gchar  *name,
 		g_set_error (error,
 			     G_FILE_ERROR,
 			     g_file_error_from_errno (save_errno),
-			     _("Directory '%s' could not be created: g_mkdir_with_parents() failed: %s"),
+			     _("Directory “%s” could not be created: g_mkdir_with_parents() failed: %s"),
 			     display_filename,
 			     g_strerror (save_errno));
 
@@ -851,7 +858,7 @@ uninstall_scheme_clicked (GtkButton              *button,
 	if (!uninstall_style_scheme (scheme))
 	{
 		gedit_warning (GTK_WINDOW (dlg),
-		               _("Could not remove color scheme \"%s\"."),
+		               _("Could not remove color scheme “%s”."),
 		               gtk_source_style_scheme_get_name (scheme));
 	}
 }
